﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace EsempioDecorator
{
    public abstract class Decorator : IComponent
    {
        private readonly IComponent _component;

        protected Decorator(IComponent component)
        {
            if (component == null)
                throw new ArgumentNullException("component");
            _component = component;
        }

        public IComponent Component
        {
            get { return _component; }
        }

        #region IComponent Members

        public abstract void Accept(IVisitor v);
        public abstract void Draw(Graphics g, float x, float y);
        public abstract SizeF Measure(Graphics g);

        #endregion
    }

    public class FilledFrame : Decorator
    {
        private SolidBrush _brush;

        public FilledFrame(IComponent component, SolidBrush brush = null)
            : base(component)
        {
            Brush = brush;
        }

        public FilledFrame(IComponent component, Color color)
            : this(component, new SolidBrush(color))
        {
        }

        public SolidBrush Brush
        {
            get
            {
                if (_brush == null)
                    _brush = new SolidBrush(Color.White);
                return _brush;
            }
            set { _brush = value; }
        }

        public override void Accept(IVisitor v)
        {
            v.Visit(this);
        }

        public override void Draw(Graphics g, float x, float y)
        {
            SizeF sizeF = Component.Measure(g);
            g.FillRectangle(Brush, x, y, sizeF.Width, sizeF.Height);
            Component.Draw(g, x, y);
        }

        public override SizeF Measure(Graphics g)
        {
            return Component.Measure(g);
        }
    }

    public class BorderedFrame : Decorator
    {
        private Pen _pen;

        public BorderedFrame(IComponent component, Pen pen = null)
            : base(component)
        {
            Pen = pen;
        }

        public BorderedFrame(IComponent component, Color color)
            : this(component, new Pen(color))
        {
        }

        public Pen Pen
        {
            get
            {
                if (_pen == null)
                    _pen = new Pen(Color.Black);
                return _pen;
            }
            set { _pen = value; }
        }

        public override void Accept(IVisitor v)
        {
            v.Visit(this);
        }

        public override void Draw(Graphics g, float x, float y)
        {
            SizeF sizeF = Component.Measure(g);
            g.DrawRectangle(Pen, x + Pen.Width / 2, y + Pen.Width / 2, sizeF.Width + Pen.Width, sizeF.Height + Pen.Width);
            Component.Draw(g, x + Pen.Width, y + Pen.Width);
        }

        public override SizeF Measure(Graphics g)
        {
            return Component.Measure(g) + new SizeF(Pen.Width * 2, Pen.Width * 2);
        }
    }

    public class RotatedFrame : Decorator
    {
        public RotatedFrame(IComponent component)
            : base(component)
        {
        }

        public override void Accept(IVisitor v)
        {
            v.Visit(this);
        }

        public override void Draw(Graphics g, float x, float y)
        {
            SizeF sizeF = Measure(g);
            GraphicsState graphicsState = g.Save();
            g.TranslateTransform(x + sizeF.Width, y);
            g.RotateTransform(90);
            Component.Draw(g, 0, 0);
            g.Restore(graphicsState);
        }

        public override SizeF Measure(Graphics g)
        {
            SizeF sizeF = Component.Measure(g);
            return new SizeF(sizeF.Height, sizeF.Width);
        }
    }
}
